//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
//
// $Id: G4OpenGLQtMovieDialog.cc,v 1.6 2008/02/15 10:54:13 lgarnier Exp $
// GEANT4 tag $Name:  $
//
// 

#define GEANT4_QT_DEBUG
#ifdef G4VIS_BUILD_OPENGLQT_DRIVER

#include "G4OpenGLQtMovieDialog.hh"
#include "G4OpenGLQtViewer.hh"

#include <qpushbutton.h>
#include <qpalette.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qfiledialog.h>
#include <qprocess.h>


// +---------------------------------------+
// +        Path for encoder               +
// +  _______                              +
// + | select| ____________________        +
// +  -------                              +
// +        Temp path                      +
// +  _______                              +
// + | select| ____________________        +
// +  -------                              +
// +                                       +
// + max number of frames  ________        +
// + ....                                  +
// +                                       +
// +     Label : X frames Saves/Encoding   +
// +         Cancel        Encode          +
// +---------------------------------------+

G4OpenGLQtMovieDialog::G4OpenGLQtMovieDialog(
 G4OpenGLQtViewer* parentViewer,
 QWidget* parentWidget
)
  : QDialog( parentWidget ),
    fParentViewer(parentViewer)
{
  setModal(false);
#if QT_VERSION < 0x040000
  setCaption( tr( " Movie parameters" ));
#else
  setWindowTitle( tr( " Movie parameters" ));
#endif


  // global layout
  QVBoxLayout* globalVLayout = new QVBoxLayout(this);
  globalVLayout->setMargin(10);
  globalVLayout->setSpacing(10);
  
  QLabel *pathLabel = new QLabel("Select path for encoder :");

#if QT_VERSION < 0x040000
  globalVLayout->add(pathLabel);
#else
  globalVLayout->addWidget(pathLabel);
#endif


  // Encoder Path 

  QWidget *selectEncoderWidget = new QWidget(this);
  QHBoxLayout *selectPathLineLayout = new QHBoxLayout(selectEncoderWidget);
  fEncoderPath = new QLineEdit("",selectEncoderWidget);
#if QT_VERSION < 0x040000
  selectPathLineLayout->add(fEncoderPath);
#else
  selectPathLineLayout->addWidget(fEncoderPath);
#endif

  QPushButton *encoderButton = new QPushButton(tr("..."),selectEncoderWidget);
  encoderButton->setMaximumWidth (30);

#if QT_VERSION < 0x040000
  selectPathLineLayout->add(encoderButton);
#else
  selectPathLineLayout->addWidget(encoderButton);
#endif

#if QT_VERSION >= 0x040000
  selectEncoderWidget->setLayout(selectPathLineLayout);
#endif
  connect( encoderButton, SIGNAL( clicked( ) ), this, SLOT(selectPathAction() ) );

#if QT_VERSION < 0x040000
  globalVLayout->add(selectEncoderWidget);
#else
  globalVLayout->addWidget(selectEncoderWidget);
#endif

  fEncoderPath->setText(fParentViewer->getEncoderPath());


  // temp folder Path 
  QWidget *selectTempFolderWidget = new QWidget(this);
  QHBoxLayout *selectEncoderLineLayout = new QHBoxLayout(selectTempFolderWidget);

  fTempPath = new QLineEdit("",selectTempFolderWidget);
#if QT_VERSION < 0x040000
  selectEncoderLineLayout->add(fTempPath);
#else
  selectEncoderLineLayout->addWidget(fTempPath);
#endif

  QPushButton *tempButton = new QPushButton(tr("..."),selectTempFolderWidget);
  tempButton->setMaximumWidth (30);

#if QT_VERSION < 0x040000
  selectEncoderLineLayout->add(tempButton);
#else
  selectEncoderLineLayout->addWidget(tempButton);
#endif

#if QT_VERSION >= 0x040000
  selectTempFolderWidget->setLayout(selectEncoderLineLayout);
#endif
  connect( tempButton, SIGNAL( clicked( ) ), this, SLOT(selectPathAction() ) );

#if QT_VERSION < 0x040000
  globalVLayout->add(selectTempFolderWidget);
#else
  globalVLayout->addWidget(selectTempFolderWidget);
#endif

  fTempPath->setText(fParentViewer->getTempFolderPath());


  // label

  QLabel *infoLabel = new QLabel("  Press SPACE to Start/Pause video recording \n  Press RETURN to Stop video recording");

#if QT_VERSION < 0x040000
  globalVLayout->add(infoLabel);
#else
  globalVLayout->addWidget(infoLabel);
#endif

  // check infos
  fCheckLabel = new QLabel("");

#if QT_VERSION < 0x040000
  globalVLayout->add(fCheckLabel);
#else
  globalVLayout->addWidget(fCheckLabel);
#endif

  // buttons
  QWidget *buttonBox = new QWidget(this);

  QHBoxLayout *buttonBoxLayout = new QHBoxLayout(buttonBox);

  QPushButton *buttonCancel = new QPushButton( tr( "&Cancel" ),buttonBox );
  buttonCancel->setAutoDefault( TRUE );
  buttonBoxLayout->addWidget(buttonCancel);

  QPushButton * buttonEncode = new QPushButton( tr( "&Apply" ),buttonBox );
  buttonEncode->setAutoDefault( TRUE );
  buttonEncode->setDefault( TRUE );
  buttonBoxLayout->addWidget(buttonEncode);

#if QT_VERSION >= 0x040000
  buttonBox->setLayout(buttonBoxLayout);
#endif
  globalVLayout->addWidget(buttonBox);



#if QT_VERSION >= 0x040000
  setLayout(globalVLayout);
#endif

  // signals and slots connections
  connect( buttonEncode, SIGNAL( clicked() ), this, SLOT( checkMovieParameters() ) );
  connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
}



G4OpenGLQtMovieDialog::~G4OpenGLQtMovieDialog()
{
}

void G4OpenGLQtMovieDialog::selectPathAction()
{
  QString nomFich =  QFileDialog::getOpenFileName ( this,
                                                    "Select your encoder",
                                                    tr("Select your encoder ...")); 
  if (nomFich == "") {
    return;
  }
  fEncoderPath->setText(nomFich);
}

/**
 * If one of parameter is incorrect, put it in red and don't valid it
 */
void G4OpenGLQtMovieDialog::checkMovieParameters() {

  fCheckLabel->setText("");
  QPalette palette( fEncoderPath->palette() );

  if (!fParentViewer->setEncoderPath(fEncoderPath->text())) {
    palette.setColor( QPalette::Base, Qt::red);
    fCheckLabel->setText ("Encoder path does not exist or file is not executable\n");
  } else {
    palette.setColor( QPalette::Base, Qt::white);
  }
  fEncoderPath->setPalette(palette);



  if (!fParentViewer->setTempFolderPath(fTempPath->text())) {
    palette.setColor( QPalette::Base, Qt::red);
    fCheckLabel->setText (fCheckLabel->text()+"Temp folder does not exist or no write access to write in\n");
  } else {
    palette.setColor( QPalette::Base, Qt::white);
  }
  fTempPath->setPalette(palette);
}

#endif
